#ifndef P1CAMERA_EXCEPTIONS_HPP
#define P1CAMERA_EXCEPTIONS_HPP

#include <stdexcept>
#include "C_P1CameraCommonErrorCodes.h"
#include "C_P1CameraCommonLinkage.h"

namespace P1
{
	namespace CameraSdk
	{
		/**
		* @brief Base exception class for all exception thrown from the C++ API
		*
		* A child class would normally call this class' constructor to extract the human readable
		* error message string. This string is then provided to the default `std::exception` class.
		*/
		class EXPORT_API SdkException : public std::runtime_error
		{
		public:
			SdkException(const SdkException& e2);	// Copy constructor - exceptions must have one such [C++ Standard, 15.1].
			SdkException(ErrorCode errorCode = kError);
			SdkException(const char* message, ErrorCode errorCode = kError);

			ErrorCode mErrorCode;
		};

		/* SDK internals */
		class EXPORT_API SdkNotOpenExc : public SdkException
		{
			public: SdkNotOpenExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API SdkNoFactoryExc : public SdkException
		{
			public: SdkNoFactoryExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API SdkFactoryErrorExc : public SdkException {
			public: SdkFactoryErrorExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API SdkNotYetImplementedExc : public SdkException {
			public: SdkNotYetImplementedExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		/* Camera connection exceptions  */

		class EXPORT_API SdkCameraNotFoundExc : public SdkException
		{
			public: SdkCameraNotFoundExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API CameraNotOpenExc : public SdkException
		{
			public:	CameraNotOpenExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API CameraCouldNotConnectExc : public SdkException
		{
			public: CameraCouldNotConnectExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		/* Properties */

		class EXPORT_API PropertyNotFoundExc : public SdkException
		{
			public: PropertyNotFoundExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API PropertyTypeNotFoundExc : public SdkException
		{
			public: PropertyTypeNotFoundExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API PropertyWrongTypeExc : public SdkException
		{
			public: PropertyWrongTypeExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API PropertySetFailedException : public SdkException
		{
			public: PropertySetFailedException(ErrorCode errorCode) : SdkException(errorCode) {}
		};

		class EXPORT_API PropertyOutOfRangeException : public SdkException
		{
			public: PropertyOutOfRangeException(ErrorCode code) : SdkException(code) {}
		};

		class EXPORT_API PropertyValueNotInListException : public SdkException
		{
			public: PropertyValueNotInListException(ErrorCode code) : SdkException(code) {}
		};

		/* Notifications */

		class EXPORT_API NotificationTimeoutExc : public SdkException
		{
			public: NotificationTimeoutExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		class EXPORT_API NotificationAlreadyExistsExc : public SdkException
		{
			public: NotificationAlreadyExistsExc(ErrorCode errorCode) : SdkException(errorCode) { }
		};

		/* Misc */

		class EXPORT_API ImageReceiveTimeoutExc : public SdkException
        {
        public: ImageReceiveTimeoutExc(ErrorCode errorCode, const char* message = nullptr) : SdkException(message, errorCode) { }
        };
		class EXPORT_API AnotherThreadPendingExc : public SdkException
		{
			public: AnotherThreadPendingExc(ErrorCode errorcode) : SdkException(errorcode) {}
		};
	}
}

#endif	// P1CAMERA_EXCEPTIONS_HPP
