#ifndef P1CAMERA_NOTIFICATIONS_HPP
#define P1CAMERA_NOTIFICATIONS_HPP

#include <P1CameraCamera.hpp>
#include "C_P1CameraCommonLinkage.h"

namespace P1
{
namespace CameraSdk
{
    class EXPORT_API Notifications
    {
    public:
        /**
         * @brief Register as listener for receiving events.
         *
         * The returned token shall be used to identify the listener in all succeeding
         * function calls.
         */
        static uint32_t RegisterListener();

        /**
         * @brief Cleans up everything for the given listener, and render the id useless.
         */
        static void DeregisterListener(const uint32_t listenerId);

        /**
         * @brief Begin listening for a camera property change
         *
         * Before any notification will arrive, you must register yourself as a listener.
         * A listener is local to the current thread. This means you must use the same thread
         * to call this function and @ref WaitForNotification.
         *
         * @param userHandle A registered listener-id.
         * @param camera The camera on which to listen for the property change
         * @param propertyId The property to listen for changes on
         */
        static void Enable(const uint32_t listenerId, const Camera& camera, const int propertyId);
        static void Enable(const uint32_t listenerId, const Camera& cameraId, const CameraEventId eventId);
        static void Disable(const uint32_t listenerId, const Camera& camera, const int propertyId);
        static void Disable(const uint32_t listenerId, const Camera& cameraId, const CameraEventId eventId);
        static void DisableAll(const uint32_t listenerId);

        /**
         * @brief Wake up all pending threads "caught" in the WaitForNotification() function.
         *
         * This function will force all pending thread to exit from @ref WaitForNotification
         * as if a timeout has arrived. Even if no timeout was configured!
         *
         * You should use this function to terminate your threads gracefully, when your app
         * exits.
         */
        static void WakeUpWaitingThreads(const uint32_t listenerId);

        /**
         * @brief Wait for the next arriving vent (property change or camera event)
         *
         * This call will block until the notification arrives, or until a defined timeout is reached.
         *
         * If timeout is reached this call throws the NotificationTimeoutExc exception.
         *
         * @param timeoutMs A millisecond timeout for when to give up waiting, or zero to wait forever.
         * @return The event that has occurred (if note timed out).
         */
        static NotificationEvent WaitForNotification(const uint32_t listenerId, uint32_t timeoutMs = 0);

        /**
         * @brief Wait for the next arriving vent (property change or camera event)
         *
         * This call will block until the notification arrives, or until a defined timeout is reached.
         *
         * @param event The event that has occurred (if note timed out)
         * @param timeoutMs A millisecond timeout for when to give up waiting, or zero to wait forever.
         * @return true is a notification arrived, false if timeout was reached
         */
        static bool WaitForNotification(const uint32_t listenerId, NotificationEvent& event, uint32_t timeoutMs = 0);
    };

} // CameraSdk
} // P1

#endif // P1CAMERA_NOTIFICATIONS_HPP
